#!/usr/bin/env python3

import os
import sys
import glob
import argparse

sys.path.append("debian/eng")
from versionlib.dotnet import SourcePackageVersion, RuntimeIdentifier


def GetSourceBuiltArtifactsTarball(
        basePath: str, 
        sdkVersion: str, 
        type: str = "packages") -> str:
    
    globPattern = ""

    if type.lower() == "packages":
        # e.g. Private.SourceBuilt.Artifacts.9.0.100-preview.7.24407.1.ubuntu.24.10-x64.tar.gz
        globPattern =  (f"{basePath}/Private.SourceBuilt.Artifacts."
                        f"{sdkVersion}*.*.tar.gz")
    elif type.lower() == "sdk":
        sdkArtifactsPath = os.path.join(basePath, "Sdk")
        if os.path.exists(sdkArtifactsPath):
            basePath = sdkArtifactsPath

        # e.g. dotnet-sdk-9.0.100-preview.7.24407.1-ubuntu.24.10-x64.tar.gz
        globPattern =  (f"{basePath}/**/dotnet-sdk-{sdkVersion}*-*.tar.gz")
    else:
        raise ValueError(f"Unknown source built artifacts tarball type '{type}'.")
    
    files = glob.glob(globPattern, recursive=True)

    if len(files) == 0:
        raise ValueError(f"Source built artifacts tarball not found "
                         f"(glob pattern: '{globPattern}').")
    elif len(files) > 1:
        raise ValueError(f"Multiple source built artifacts artifacts tarballs "
                         f"found (glob pattern: '{globPattern}').")

    return files[0]


if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--type",
                        type=str,
                        dest="Type",
                        choices=["packages","sdk"],
                        required=True,
                        metavar="TYPE",
                        help = "artifact type")
    parser.add_argument("--base-path",
                        type=str,
                        dest="BasePath",
                        required=True,
                        metavar="PATH",
                        help="base path where the artifacts tarball is "
                             "located")
    args = parser.parse_args()

    version = SourcePackageVersion.ParseFromChangelog(
        os.path.join("debian", "changelog"))

    print(GetSourceBuiltArtifactsTarball(
        basePath=args.BasePath,
        sdkVersion=str(version.SdkVersion),
        type=args.Type))



import enum
from ._version import get_versions
from ._name_mangler import _NameMangler

__version__ = get_versions()["version"]
__version_info__ = tuple(int(n) for n in __version__.partition("+")[0].split("."))
del get_versions

_name_mangler = _NameMangler()

# The first argument to the `_generate_next_value_` function of the `enum.Enum`
# class is documented to be the name of the enum member, not the enum class:
#
#     https://docs.python.org/3.6/library/enum.html#using-automatic-values
#
# Pylint, though, doesn't know about this so we need to disable it's check for
# `self` arguments.
# pylint: disable=no-self-argument


class StrEnum(str, enum.Enum):
    """
    StrEnum is a Python ``enum.Enum`` that inherits from ``str``. The default
    ``auto()`` behavior uses the member name as its value.

    Example usage::

        class Example(StrEnum):
            UPPER_CASE = auto()
            lower_case = auto()
            MixedCase = auto()

        assert Example.UPPER_CASE == "UPPER_CASE"
        assert Example.lower_case == "lower_case"
        assert Example.MixedCase == "MixedCase"
    """

    def __new__(cls, value, *args, **kwargs):
        if not isinstance(value, (str, enum.auto)):
            raise TypeError(
                f"Values of StrEnums must be strings: {value!r} is a {type(value)}"
            )
        return super().__new__(cls, value, *args, **kwargs)

    def __str__(self):
        return str(self.value)

    def _generate_next_value_(name, *_):
        return name


class LowercaseStrEnum(StrEnum):
    """
    A ``StrEnum`` where ``auto()`` will convert the name to `lowercase` to
    produce each member's value.

    Example usage::

        class Example(LowercaseStrEnum):
            UPPER_CASE = auto()
            lower_case = auto()
            MixedCase = auto()

        assert Example.UPPER_CASE == "upper_case"
        assert Example.lower_case == "lower_case"
        assert Example.MixedCase == "mixedcase"

    .. versionadded:: 0.4.3
    """

    def _generate_next_value_(name, *_):
        return name.lower()


class UppercaseStrEnum(StrEnum):
    """
    A ``StrEnum`` where ``auto()`` will convert the name to `UPPERCASE` to
    produce each member's value.

    Example usage::

        class Example(UppercaseStrEnum):
            UPPER_CASE = auto()
            lower_case = auto()
            MixedCase = auto()

        assert Example.UPPER_CASE == "UPPER_CASE"
        assert Example.lower_case == "LOWER_CASE"
        assert Example.MixedCase == "MIXEDCASE"

    .. versionadded:: 0.4.3
    """

    def _generate_next_value_(name, *_):
        return name.upper()


class CamelCaseStrEnum(StrEnum):
    """
    A ``StrEnum`` where ``auto()`` will convert the name to `camelCase` to
    produce each member's value.

    Example usage::

        class Example(CamelCaseStrEnum):
            UPPER_CASE = auto()
            lower_case = auto()
            MixedCase = auto()

        assert Example.UPPER_CASE == "upperCase"
        assert Example.lower_case == "lowerCase"
        assert Example.MixedCase == "mixedCase"

    .. versionadded:: 0.4.5
    """

    def _generate_next_value_(name, *_):
        return _name_mangler.camel(name)


class PascalCaseStrEnum(StrEnum):
    """
    A ``StrEnum`` where ``auto()`` will convert the name to `PascalCase` to
    produce each member's value.

    Example usage::

        class Example(PascalCaseStrEnum):
            UPPER_CASE = auto()
            lower_case = auto()
            MixedCase = auto()

        assert Example.UPPER_CASE == "UpperCase"
        assert Example.lower_case == "LowerCase"
        assert Example.MixedCase == "MixedCase"

    .. versionadded:: 0.4.5
    """

    def _generate_next_value_(name, *_):
        return _name_mangler.pascal(name)


class KebabCaseStrEnum(StrEnum):
    """
    A ``StrEnum`` where ``auto()`` will convert the name to `kebab-case` to
    produce each member's value.

    Example usage::

        class Example(KebabCaseStrEnum):
            UPPER_CASE = auto()
            lower_case = auto()
            MixedCase = auto()

        assert Example.UPPER_CASE == "upper-case"
        assert Example.lower_case == "lower-case"
        assert Example.MixedCase == "mixed-case"

    .. versionadded:: 0.4.5
    """

    def _generate_next_value_(name, *_):
        return _name_mangler.kebab(name)


class SnakeCaseStrEnum(StrEnum):
    """
    A ``StrEnum`` where ``auto()`` will convert the name to `snake_case` to
    produce each member's value.

    Example usage::

        class Example(SnakeCaseStrEnum):
            UPPER_CASE = auto()
            lower_case = auto()
            MixedCase = auto()

        assert Example.UPPER_CASE == "upper_case"
        assert Example.lower_case == "lower_case"
        assert Example.MixedCase == "mixed_case"

    .. versionadded:: 0.4.5
    """

    def _generate_next_value_(name, *_):
        return _name_mangler.snake(name)


class MacroCaseStrEnum(StrEnum):
    """
    A ``StrEnum`` where ``auto()`` will convert the name to `MACRO_CASE` to
    produce each member's value.

    Example usage::

        class Example(MacroCaseStrEnum):
            UPPER_CASE = auto()
            lower_case = auto()
            MixedCase = auto()

        assert Example.UPPER_CASE == "UPPER_CASE"
        assert Example.lower_case == "LOWER_CASE"
        assert Example.MixedCase == "MIXED_CASE"

    .. versionadded:: 0.4.6
    """

    def _generate_next_value_(name, *_):
        return _name_mangler.macro(name)


class CamelSnakeCaseStrEnum(StrEnum):
    """
    A ``StrEnum`` where ``auto()`` will convert the name to `camel_Snake_Case` to
    produce each member's value.

    Example usage::

        class Example(CamelSnakeCaseStrEnum):
            UPPER_CASE = auto()
            lower_case = auto()
            MixedCase = auto()

        assert Example.UPPER_CASE == "upper_Case"
        assert Example.lower_case == "lower_Case"
        assert Example.MixedCase == "mixed_Case"

    .. versionadded:: 0.4.8
    """

    def _generate_next_value_(name, *_):
        return _name_mangler.camel_snake(name)


class PascalSnakeCaseStrEnum(StrEnum):
    """
    A ``StrEnum`` where ``auto()`` will convert the name to `Pascal_Snake_Case` to
    produce each member's value.

    Example usage::

        class Example(PascalSnakeCaseStrEnum):
            UPPER_CASE = auto()
            lower_case = auto()
            MixedCase = auto()

        assert Example.UPPER_CASE == "Upper_Case"
        assert Example.lower_case == "Lower_Case"
        assert Example.MixedCase == "Mixed_Case"

    .. versionadded:: 0.4.8
    """

    def _generate_next_value_(name, *_):
        return _name_mangler.pascal_snake(name)


class SpongebobCaseStrEnum(StrEnum):
    """
    A ``StrEnum`` where ``auto()`` will convert the name to `SpONGEBob_CAse` to
    produce each member's value.

    Example usage::

        class Example(SpongebobCaseStrEnum):
            UPPER_CASE = auto()
            lower_case = auto()
            MixedCase = auto()

        assert Example.UPPER_CASE == "uPpER_cAsE"
        assert Example.lower_case == "lowER_CASe"
        assert Example.MixedCase == "MixeD_CAse"

    .. versionadded:: 0.4.8
    """

    def _generate_next_value_(name, *_):
        return _name_mangler.spongebob(name)


class CobolCaseStrEnum(StrEnum):
    """
    A ``StrEnum`` where ``auto()`` will convert the name to `COBOL-CASE` to
    produce each member's value.

    Example usage::

        class Example(CobolCaseStrEnum):
            UPPER_CASE = auto()
            lower_case = auto()
            MixedCase = auto()

        assert Example.UPPER_CASE == "UPPER-CASE"
        assert Example.lower_case == "LOWER-CASE"
        assert Example.MixedCase == "MIXED-CASE"

    .. versionadded:: 0.4.8
    """

    def _generate_next_value_(name, *_):
        return _name_mangler.cobol(name)


class HttpHeaderCaseStrEnum(StrEnum):
    """
    A ``StrEnum`` where ``auto()`` will convert the name to `Http-Header-Case` to
    produce each member's value.

    Example usage::

        class Example(HttpHeaderCaseStrEnum):
            UPPER_CASE = auto()
            lower_case = auto()
            MixedCase = auto()

        assert Example.UPPER_CASE == "Upper-Case"
        assert Example.lower_case == "Lower-Case"
        assert Example.MixedCase == "Mixed-Case"

    .. versionadded:: 0.4.8
    """

    def _generate_next_value_(name, *_):
        return _name_mangler.http_header(name)



from strenum import StrEnum
import platform
import re

from . import dpkg


class PreReleaseType(StrEnum):
    Alpha = "alpha"
    Beta = "beta"
    Preview = "preview"
    ReleaseCandidate = "rc"


class PrereleaseVersion:
    def __init__(self,
                 type: PreReleaseType,
                 revision: int | None,
                 buildMetadata: list[str] | None) -> None:
        self.Type = type
        self.Revision = revision
        self.BuildMetadata = buildMetadata

    def __str__(self) -> str:
        prereleaseVersion = str(self.Type)

        if self.Revision is not None:
            prereleaseVersion += "." + str(self.Revision)

        if self.BuildMetadata is not None:
            for buildMetadataPart in self.BuildMetadata:
                prereleaseVersion += "." + buildMetadataPart

        return prereleaseVersion

    def DebRepresentation(self) -> str:
        prereleaseVersion = str(self.Type)

        if self.Revision is not None:
            prereleaseVersion += str(self.Revision)

        if self.BuildMetadata is not None:
            for buildMetadataPart in self.BuildMetadata:
                prereleaseVersion += "." + buildMetadataPart

        return prereleaseVersion

    def __repr__(self) -> str:
        return str(self)

    def __eq__(self, __value: object) -> bool:
        if __value is None:
            return False

        if self.Type != __value.Type:
            return False

        if self.Revision != __value.Revision:
            return False

        if (self.BuildMetadata is None) != (__value.BuildMetadata is None):
            return False
        elif self.BuildMetadata is None:
            return True
        elif len(self.BuildMetadata) != len(__value.BuildMetadata):
            return False

        for i in range(len(self.BuildMetadata)):
            if self.BuildMetadata[i] != __value.BuildMetadata[i]:
                return False

        return True


class SdkVersion:
    def __init__(self,
                 major: int,
                 minor: int,
                 featureBand: int,
                 patch: int,
                 preReleaseVersion: PrereleaseVersion) -> None:
        if major < 1:
            raise ValueError(".NET SDK major version number can't be "
                             f"non-positive (major = {major}).")
        if minor < 0:
            raise ValueError(".NET SDK minor version number can't be "
                             f"negative (minor = {minor}).")
        if featureBand < 100 or featureBand > 900 or (featureBand % 100) != 0:
            raise ValueError(".NET SDK feature band has to be a "
                             "multiple of 100 between (inclusive) 100 and 900 "
                             f"(featureBand = {featureBand}).")
        if patch < 0 or patch > 99:
            raise ValueError(".NET SDK patch number has to be "
                             "between (inclusive) 0 and 99 "
                             f"(patch = {patch}).")

        self.Major = major
        self.Minor = minor
        self.FeatureBand = featureBand
        self.Patch = patch
        self.Revision = featureBand + patch
        self.PreReleaseVersion = preReleaseVersion

    def __str__(self) -> str:
        sdkVersion = f"{self.Major}.{self.Minor}.{self.Revision}"

        if self.PreReleaseVersion is not None:
            sdkVersion += "-" + str(self.PreReleaseVersion)

        return sdkVersion

    def DebRepresentation(self) -> str:
        sdkVersion = f"{self.Major}.{self.Minor}.{self.Revision}"

        if self.PreReleaseVersion is not None:
            sdkVersion += "~" + self.PreReleaseVersion.DebRepresentation()

        return sdkVersion

    def __repr__(self) -> str:
        return str(self)

    def __eq__(self, other: 'SdkVersion') -> bool:
        if not isinstance(other, SdkVersion):
            return False

        return (self.Major == other.Major
                and self.Minor == other.Minor
                and self.Minor == other.Minor
                and self.FeatureBand == other.FeatureBand
                and self.Patch == other.Patch
                and self.PreReleaseVersion == other.PreReleaseVersion)


class RuntimeVersion:
    def __init__(self,
                 major: int,
                 minor: int,
                 patch: int,
                 preReleaseVersion: PrereleaseVersion) -> None:
        if major < 1:
            raise ValueError(".NET Runtime major version number can't be "
                             f"non-positive (major = {major}).")
        if minor < 0:
            raise ValueError(".NET Runtime minor version number can't be "
                             f"negative (minor = {minor}).")
        if patch < 0 or patch > 99:
            raise ValueError(".NET Runtime patch number has to be "
                             f"between 0 and 99 (patch = {patch}).")

        self.Major = major
        self.Minor = minor
        self.Patch = patch
        self.PreReleaseVersion = preReleaseVersion

    def __str__(self) -> str:
        runtimeVersion = f"{self.Major}.{self.Minor}.{self.Patch}"

        if self.PreReleaseVersion is not None:
            runtimeVersion += "-" + str(self.PreReleaseVersion)

        return runtimeVersion

    def DebRepresentation(self) -> str:
        runtimeVersion = f"{self.Major}.{self.Minor}.{self.Patch}"

        if self.PreReleaseVersion is not None:
            runtimeVersion += "~" + self.PreReleaseVersion.DebRepresentation()

        return runtimeVersion

    def __repr__(self) -> str:
        return str(self)

    def __eq__(self, other: 'RuntimeVersion') -> bool:
        if not isinstance(other, RuntimeVersion):
            return False

        return (self.Major == other.Major
                and self.Minor == other.Minor
                and self.Patch == other.Patch
                and self.PreReleaseVersion == other.PreReleaseVersion)


class SourcePackageVersion:
    Pattern = (r"\A(?:(?P<OverwrittenVersion>.+)\+really)?"
               r"(?P<Major>[1-9]\d*)\."
               r"(?P<Minor>\d+)\."
               r"(?P<SdkFeatureBand>[1-9])"
               r"(?P<SdkPatch>\d\d)"
               r"(?:-"
                    r"(?P=Major)\."  # noqa: E127
                    r"(?P=Minor)\."
                    r"(?P<RuntimePatch>0|[1-9]\d?)"
               r")?"
               r"(?:~"
                    r"(?P<PreReleaseType>rc|preview|beta|alpha)"
                    r"(?P<PreReleaseRevision>[1-9]\d*)"
                    r"(?P<BuildMetadata>(?:\.[0-9a-z]+)+)?"
               r")?"
               r"(?:~bootstrap\+"
                    r"(?P<BootstrapArch>amd64|arm64|s390x|ppc64el|riscv64)"
               r")?"
               r"(?:-"
                    r"(?P<DebianRevision>[A-Za-z0-9.+:~]+)"
                    r"(?:ubuntu(?P<UbuntuRevision>[A-Za-z0-9.+:~]+))"
               r")\Z")

    def __init__(self,
                 sdkVersion: str,
                 runtimeVersion: str,
                 debRevision: str,
                 raw: str,
                 isFO127Compliant: bool,
                 bootstrapArch: str | None = None,
                 overwrittenVersion: str | None = None) -> None:
        self.SdkVersion = sdkVersion
        self.RuntimeVersion = runtimeVersion
        self.DebRevision = debRevision
        self.Raw = raw
        self.IsFO127Compliant = isFO127Compliant
        self.BootstrapArchitecture = bootstrapArch
        self.OverwrittenVersion = overwrittenVersion

    def RuntimeOnlyDebVersion(self, buildSuffix: str = "") -> str:
        versionString = self.RuntimeVersion.DebRepresentation() + buildSuffix

        if (self.BootstrapArchitecture is not None):
            versionString += f"~bootstrap+{self.BootstrapArchitecture}"

        versionString += f"-{self.DebRevision}"
        return versionString

    def SdkOnlyDebVersion(self, buildSuffix: str = "") -> str:
        versionString = self.SdkVersion.DebRepresentation() + buildSuffix

        if (self.BootstrapArchitecture is not None):
            versionString += f"~bootstrap+{self.BootstrapArchitecture}"

        versionString += f"-{self.DebRevision}"
        return versionString

    @staticmethod
    def Parse(value: str) -> 'SourcePackageVersion':
        parsingResult = re.search(SourcePackageVersion.Pattern, value)

        if parsingResult is None:
            raise ValueError("The format of the provided .NET source package "
                             "version identifier could not be recognized "
                             f"(value=\"{value}\").")

        if parsingResult.group("PreReleaseType") is not None:
            preReleaseType = parsingResult.group("PreReleaseType")
            preReleaseRevision = parsingResult.group("PreReleaseRevision")
            buildMetadata = parsingResult.group("BuildMetadata")

            if preReleaseRevision is not None:
                preReleaseRevision = int(preReleaseRevision)

            if buildMetadata is not None:
                buildMetadata = buildMetadata.split('.')[1:]

            preReleaseVersion = PrereleaseVersion(
                preReleaseType, preReleaseRevision, buildMetadata)
        else:
            preReleaseVersion = None

        major = int(parsingResult.group("Major"))
        minor = int(parsingResult.group("Minor"))
        sdkFeatureBand = int(parsingResult.group("SdkFeatureBand")) * 100
        sdkPatch = int(parsingResult.group("SdkPatch"))
        sdkVersion = SdkVersion(
            major, minor, sdkFeatureBand, sdkPatch, preReleaseVersion)

        if parsingResult.group("RuntimePatch") is not None:
            isFO127Compliant = True
            runtimePatch = int(parsingResult.group("RuntimePatch"))
        else:
            isFO127Compliant = False
            runtimePatch = sdkPatch

        runtimeVersion = RuntimeVersion(
            major, minor, runtimePatch, preReleaseVersion)

        overwrittenVersion = parsingResult.group("OverwrittenVersion")
        bootstrapArch = parsingResult.group("BootstrapArch")
        debianRevision = parsingResult.group("DebianRevision")
        ubuntuRevision = parsingResult.group("UbuntuRevision")
        debRevision = dpkg.DebRevision(debianRevision, ubuntuRevision)

        return SourcePackageVersion(
            sdkVersion,
            runtimeVersion,
            debRevision,
            value,
            isFO127Compliant,
            bootstrapArch,
            overwrittenVersion)

    @staticmethod
    def ParseFromChangelog(path: str) -> 'SourcePackageVersion':
        version = dpkg.Changelog(path).ParseFieldOfLatestEntry(
            dpkg.ChangelogEntryField.VERSION)

        return SourcePackageVersion.Parse(version)

    def __str__(self) -> str:
        return self.Raw


class OperatingSystemIdentifier:
    def __init__(self, name, version):
        self.Name = name
        self.Version = version

    @staticmethod
    def Current() -> 'OperatingSystemIdentifier':
        osReleaseData = platform.freedesktop_os_release()

        osId = OperatingSystemIdentifier(
            osReleaseData["ID"],
            osReleaseData["VERSION_ID"])

        return osId

    def __str__(self) -> str:
        return f"{self.Name}.{self.Version}"


class ArchitectureIdentifier(StrEnum):
    X64 = "x64"
    ARM64 = "arm64"
    S390X = "s390x"
    PPC64LE = "ppc64le"

    @staticmethod
    def ParseMachineIdentifier(machineIdentifier: str) -> 'ArchitectureIdentifier':  # noqa: E501
        match machineIdentifier:
            case "x86_64":
                return ArchitectureIdentifier.X64
            case "aarch64":
                return ArchitectureIdentifier.ARM64
            case "s390x":
                return ArchitectureIdentifier.S390X
            case "ppc64le":
                return ArchitectureIdentifier.PPC64LE
            case _:
                raise RuntimeError(f"{machineIdentifier} is an "
                                   "unknown/unsupported system architecture.")

    @staticmethod
    def Current() -> 'ArchitectureIdentifier':
        machineIdentifier = platform.machine()
        return ArchitectureIdentifier.ParseMachineIdentifier(machineIdentifier)


class RuntimeIdentifier:
    def __init__(self,
                 osIdentifier: OperatingSystemIdentifier,
                 architecture: ArchitectureIdentifier | None) -> None:
        self.OperatingSystemIdentifier = osIdentifier
        self.ArchitectureIdentifier = architecture

    @staticmethod
    def FromPlatformData() -> 'RuntimeIdentifier':
        return RuntimeIdentifier(
            OperatingSystemIdentifier.Current(),
            ArchitectureIdentifier.Current())

    def __str__(self) -> str:
        rid = str(self.OperatingSystemIdentifier)

        if self.ArchitectureIdentifier is not None:
            rid += '-' + str(self.ArchitectureIdentifier)

        return rid



import unittest
import sys
import os

sys.path.append("debian/eng")
from versionlib.dotnet import RuntimeVersion, SdkVersion, SourcePackageVersion, PrereleaseVersion, PreReleaseType  # noqa: E402, E501


class TestSourcePackageVersionParsing(unittest.TestCase):
    def test_When_ParsingFO127NonCompliantDotnetSourcePackageVersion1_Should_BeRecognizedCorrectly(self):  # noqa: E501
        version = SourcePackageVersion.Parse("7.0.100-0ubuntu1")

        self.assertEqual(version.Raw, "7.0.100-0ubuntu1")
        self.assertEqual(version.IsFO127Compliant, False)

        self.assertEqual(version.SdkVersion.Major, 7)
        self.assertEqual(version.SdkVersion.Minor, 0)
        self.assertEqual(version.SdkVersion.FeatureBand, 100)
        self.assertEqual(version.SdkVersion.Patch, 0)
        self.assertEqual(version.SdkVersion.PreReleaseVersion, None)
        self.assertEqual(version.SdkVersion, SdkVersion(7,0,100,0, None))
        self.assertEqual(version.SdkVersion.DebRepresentation(), "7.0.100")

        self.assertEqual(version.RuntimeVersion.Major, 7)
        self.assertEqual(version.RuntimeVersion.Minor, 0)
        self.assertEqual(version.RuntimeVersion.Patch, 0)
        self.assertEqual(version.RuntimeVersion.PreReleaseVersion, None)
        self.assertEqual(version.RuntimeVersion, RuntimeVersion(7,0,0, None))
        self.assertEqual(version.RuntimeVersion.DebRepresentation(), "7.0.0")

        self.assertEqual(version.DebRevision.DebianRevision, "0")
        self.assertEqual(version.DebRevision.UbuntuRevision, "1")
        self.assertEqual(version.BootstrapArchitecture, None)
        self.assertEqual(version.OverwrittenVersion, None)

    def test_When_ParsingFO127NonCompliantDotnetSourcePackageVersion2_Should_BeRecognizedCorrectly(self):  # noqa: E501
        version = SourcePackageVersion.Parse(
            "999.999.999~preview999.999-999~prefix999ubuntu999~postfix999")

        self.assertEqual(
            version.Raw,
            "999.999.999~preview999.999-999~prefix999ubuntu999~postfix999")
        self.assertEqual(version.IsFO127Compliant, False)

        self.assertEqual(version.SdkVersion.Major, 999)
        self.assertEqual(version.SdkVersion.Minor, 999)
        self.assertEqual(version.SdkVersion.FeatureBand, 900)
        self.assertEqual(version.SdkVersion.Patch, 99)
        self.assertEqual(version.SdkVersion.PreReleaseVersion.Type,
                         PreReleaseType.Preview)
        self.assertEqual(version.SdkVersion.PreReleaseVersion.Revision, 999)
        self.assertEqual(version.SdkVersion.PreReleaseVersion.BuildMetadata,
                         ["999"])
        self.assertEqual(version.SdkVersion, SdkVersion(999,999,900,99, PrereleaseVersion(PreReleaseType.Preview, 999, ["999"])))
        self.assertEqual(version.SdkOnlyDebVersion(), "999.999.999~preview999.999-999~prefix999ubuntu999~postfix999")
        self.assertEqual(version.SdkOnlyDebVersion(buildSuffix="+build1"), "999.999.999~preview999.999+build1-999~prefix999ubuntu999~postfix999")

        self.assertEqual(version.RuntimeVersion.Major, 999)
        self.assertEqual(version.RuntimeVersion.Minor, 999)
        self.assertEqual(version.RuntimeVersion.Patch, 99)
        self.assertEqual(version.RuntimeVersion.PreReleaseVersion.Type,
                         PreReleaseType.Preview)
        self.assertEqual(version.RuntimeVersion.PreReleaseVersion.Revision,
                         999)
        self.assertEqual(version
                         .RuntimeVersion.PreReleaseVersion.BuildMetadata,
                         ["999"])
        self.assertEqual(version.RuntimeVersion, RuntimeVersion(999,999,99, PrereleaseVersion(PreReleaseType.Preview, 999, ["999"])))
        self.assertEqual(version.RuntimeOnlyDebVersion(), "999.999.99~preview999.999-999~prefix999ubuntu999~postfix999")
        self.assertEqual(version.RuntimeOnlyDebVersion(buildSuffix="+build1"), "999.999.99~preview999.999+build1-999~prefix999ubuntu999~postfix999")

        self.assertEqual(version.DebRevision.DebianRevision, "999~prefix999")
        self.assertEqual(version.DebRevision.UbuntuRevision, "999~postfix999")
        self.assertEqual(version.BootstrapArchitecture, None)
        self.assertEqual(version.OverwrittenVersion, None)

    def test_When_ParsingFO127CompliantDotnetSourcePackageVersion1_Should_BeRecognizedCorrectly(self):  # noqa: E501
        version = SourcePackageVersion.Parse(
            "8.0.100-8.0.0~rc1-0ubuntu1")

        self.assertEqual(version.Raw, "8.0.100-8.0.0~rc1-0ubuntu1")
        self.assertEqual(version.IsFO127Compliant, True)

        self.assertEqual(version.SdkVersion.Major, 8)
        self.assertEqual(version.SdkVersion.Minor, 0)
        self.assertEqual(version.SdkVersion.FeatureBand, 100)
        self.assertEqual(version.SdkVersion.Patch, 0)
        self.assertEqual(version.SdkVersion.PreReleaseVersion.Type,
                         PreReleaseType.ReleaseCandidate)
        self.assertEqual(version.SdkVersion.PreReleaseVersion.Revision, 1)
        self.assertEqual(version.SdkVersion.PreReleaseVersion.BuildMetadata,
                         None)
        self.assertEqual(version.SdkVersion, SdkVersion(8, 0, 100, 0, PrereleaseVersion(PreReleaseType.ReleaseCandidate, 1, None)))
        self.assertEqual(version.SdkOnlyDebVersion(), "8.0.100~rc1-0ubuntu1")
        self.assertEqual(version.SdkOnlyDebVersion(buildSuffix="+build1"), "8.0.100~rc1+build1-0ubuntu1")

        self.assertEqual(version.RuntimeVersion.Major, 8)
        self.assertEqual(version.RuntimeVersion.Minor, 0)
        self.assertEqual(version.RuntimeVersion.Patch, 0)
        self.assertEqual(version.RuntimeVersion.PreReleaseVersion.Type,
                         PreReleaseType.ReleaseCandidate)
        self.assertEqual(version.RuntimeVersion.PreReleaseVersion.Revision, 1)
        self.assertEqual(version
                         .RuntimeVersion.PreReleaseVersion.BuildMetadata, None)
        self.assertEqual(version.RuntimeVersion, RuntimeVersion(8, 0, 0, PrereleaseVersion(PreReleaseType.ReleaseCandidate, 1, None)))
        self.assertEqual(version.RuntimeOnlyDebVersion(), "8.0.0~rc1-0ubuntu1")
        self.assertEqual(version.RuntimeOnlyDebVersion(buildSuffix="+build1"), "8.0.0~rc1+build1-0ubuntu1")

        self.assertEqual(version.DebRevision.DebianRevision, "0")
        self.assertEqual(version.DebRevision.UbuntuRevision, "1")
        self.assertEqual(version.BootstrapArchitecture, None)
        self.assertEqual(version.OverwrittenVersion, None)

    def test_When_ParsingDotnetSourcePackageVersionFromChangelog_Should_BeRecognizedCorrectly(self):   # noqa: E501
        testRoot = os.path.dirname(os.path.realpath(__file__))

        version = SourcePackageVersion.ParseFromChangelog(
            os.path.join(testRoot, "changelog.mock"))

        self.assertEqual(version.Raw, "8.0.100-8.0.0~rc1-0ubuntu1~test1")
        self.assertEqual(version.IsFO127Compliant, True)

        self.assertEqual(version.SdkVersion.Major, 8)
        self.assertEqual(version.SdkVersion.Minor, 0)
        self.assertEqual(version.SdkVersion.FeatureBand, 100)
        self.assertEqual(version.SdkVersion.Patch, 0)
        self.assertEqual(version.SdkVersion.PreReleaseVersion.Type,
                         PreReleaseType.ReleaseCandidate)
        self.assertEqual(version.SdkVersion.PreReleaseVersion.Revision, 1)
        self.assertEqual(version.SdkVersion.PreReleaseVersion.BuildMetadata,
                         None)
        self.assertEqual(version.SdkVersion, SdkVersion(8, 0, 100, 0, PrereleaseVersion(PreReleaseType.ReleaseCandidate, 1, None)))
        self.assertEqual(version.SdkOnlyDebVersion(), "8.0.100~rc1-0ubuntu1~test1")
        self.assertEqual(version.SdkOnlyDebVersion(buildSuffix="+build1"), "8.0.100~rc1+build1-0ubuntu1~test1")

        self.assertEqual(version.RuntimeVersion.Major, 8)
        self.assertEqual(version.RuntimeVersion.Minor, 0)
        self.assertEqual(version.RuntimeVersion.Patch, 0)
        self.assertEqual(version.RuntimeVersion.PreReleaseVersion.Type,
                         PreReleaseType.ReleaseCandidate)
        self.assertEqual(version.RuntimeVersion.PreReleaseVersion.Revision, 1)
        self.assertEqual(version
                         .RuntimeVersion.PreReleaseVersion.BuildMetadata, None)
        self.assertEqual(version.RuntimeVersion, RuntimeVersion(8, 0, 0, PrereleaseVersion(PreReleaseType.ReleaseCandidate, 1, None)))
        self.assertEqual(version.RuntimeOnlyDebVersion(), "8.0.0~rc1-0ubuntu1~test1")
        self.assertEqual(version.RuntimeOnlyDebVersion(buildSuffix="+build1"), "8.0.0~rc1+build1-0ubuntu1~test1")

        self.assertEqual(version.DebRevision.DebianRevision, "0")
        self.assertEqual(version.DebRevision.UbuntuRevision, "1~test1")
        self.assertEqual(version.BootstrapArchitecture, None)
        self.assertEqual(version.OverwrittenVersion, None)

    def test_When_ParsingChangelogOfCurrentPackage_Should_RaiseNoError(self):   # noqa: E501
        testRoot = os.path.dirname(os.path.realpath(__file__))

        SourcePackageVersion.ParseFromChangelog(
            os.path.join(testRoot, os.pardir, os.pardir, "changelog"))

    def test_When_ParsingOverwrittenVersionString_Should_BeRecognizedCorrectly(self):  # noqa: E501
        version = SourcePackageVersion.Parse(
            "8.0.100-8.0.0~rc1+really7.0.100-7.0.0~beta1~bootstrap+amd64-0ubuntu1")  # noqa: E501

        self.assertEqual(version.Raw, "8.0.100-8.0.0~rc1+really7.0.100-7.0.0~beta1~bootstrap+amd64-0ubuntu1")  # noqa: E501
        self.assertEqual(version.IsFO127Compliant, True)

        self.assertEqual(version.SdkVersion.Major, 7)
        self.assertEqual(version.SdkVersion.Minor, 0)
        self.assertEqual(version.SdkVersion.FeatureBand, 100)
        self.assertEqual(version.SdkVersion.Patch, 0)
        self.assertEqual(version.SdkVersion.PreReleaseVersion.Type,
                         PreReleaseType.Beta)
        self.assertEqual(version.SdkVersion.PreReleaseVersion.Revision, 1)
        self.assertEqual(version.SdkVersion.PreReleaseVersion.BuildMetadata,
                         None)
        self.assertEqual(version.SdkVersion, SdkVersion(7, 0, 100, 0, PrereleaseVersion(PreReleaseType.Beta, 1, None)))
        self.assertEqual(version.SdkOnlyDebVersion(), "7.0.100~beta1~bootstrap+amd64-0ubuntu1")
        self.assertEqual(version.SdkOnlyDebVersion(buildSuffix="+build1"), "7.0.100~beta1+build1~bootstrap+amd64-0ubuntu1")

        self.assertEqual(version.RuntimeVersion.Major, 7)
        self.assertEqual(version.RuntimeVersion.Minor, 0)
        self.assertEqual(version.RuntimeVersion.Patch, 0)
        self.assertEqual(version.RuntimeVersion.PreReleaseVersion.Type,
                         PreReleaseType.Beta)
        self.assertEqual(version.RuntimeVersion.PreReleaseVersion.Revision, 1)
        self.assertEqual(version
                         .RuntimeVersion.PreReleaseVersion.BuildMetadata, None)
        self.assertEqual(version.RuntimeVersion, RuntimeVersion(7, 0, 0, PrereleaseVersion(PreReleaseType.Beta, 1, None)))
        self.assertEqual(version.RuntimeOnlyDebVersion(), "7.0.0~beta1~bootstrap+amd64-0ubuntu1")
        self.assertEqual(version.RuntimeOnlyDebVersion(buildSuffix="+build1"), "7.0.0~beta1+build1~bootstrap+amd64-0ubuntu1")

        self.assertEqual(version.DebRevision.DebianRevision, "0")
        self.assertEqual(version.DebRevision.UbuntuRevision, "1")
        self.assertEqual(version.BootstrapArchitecture, "amd64")
        self.assertEqual(version.OverwrittenVersion, "8.0.100-8.0.0~rc1")


if __name__ == '__main__':
    unittest.main()



from typing import Tuple, Any

class Comparable:
    def __eq__(self, other: Any) -> bool: ...
    def __ne__(self, other: Any) -> bool: ...
    def __lt__(self, other: Any) -> bool: ...
    def __le__(self, other: Any) -> bool: ...
    def __gt__(self, other: Any) -> bool: ...
    def __ge__(self, other: Any) -> bool: ...
    def _cmp_values(self, other: Any) -> Tuple[str, str]: ...
